-----------------------------------------------------------------------------------------------
-- Client Lua Script for AutoFollowHelper
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChatSystemLib"
require "GameLib" 

-----------------------------------------------------------------------------------------------
-- AutoFollowHelper Module Definition
-----------------------------------------------------------------------------------------------
local AutoFollowHelper = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function AutoFollowHelper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function AutoFollowHelper:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- AutoFollowHelper OnLoad
-----------------------------------------------------------------------------------------------
function AutoFollowHelper:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("AutoFollowHelper.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- AutoFollowHelper OnDocLoaded
-----------------------------------------------------------------------------------------------
function AutoFollowHelper:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "AutoFollowHelperForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		local lock = true
	    self.wndMain:Show(false, true)
		self.wndMain:SetStyle("Moveable", lock)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("afshow", "OnShow",self)
		Apollo.RegisterSlashCommand("afhide", "OnHide", self)
		Apollo.RegisterEventHandler("WindowManagementReady", 			"OnWindowManagementReady", self)
		Apollo.RegisterEventHandler("TargetUnitChanged", "OnTargetUnitChanged", self)
		
		self.timerDelay = ApolloTimer.Create(1, false, "OnDelayTimer", self)


		-- Do additional Addon initialization here
		ChatSystemLib.PostOnChannel(2,"AutoFollowHelper loaded.  \nClick a target and push the button to follow them. \nUse /afshow and /afhide to show and hide the window.")

	end
end

-----------------------------------------------------------------------------------------------
-- AutoFollowHelper Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function AutoFollowHelper:OnHide()
	if self.wndMain:IsShown() == true then
		self.wndMain:Show(false,true)
	end
end

function AutoFollowHelper:OnShow()
	if self.wndMain:IsShown() == false then
		self.wndMain:Show(true,true)
	end
end

function AutoFollowHelper:OnDelayTimer()
	if GameLib.GetTargetUnit() then 
		self.timerDelay = nil 
		self:OnShow()
	
	else self.timerDelay:Start()
	end
end

function AutoFollowHelper:OnTargetUnitChanged()
	if GameLib.GetTargetUnit() then
		if not self.wndMain:IsShown() then
			self.wndMain:Show(true,true)
		end
	elseif not GameLib.GetTargetUnit() then
		if self.wndMain:IsShown() then
			self.wndMain:Show(false,true)
		end
	end
end


-----------------------------------------------------------------------------------------------
-- AutoFollowHelperForm Functions
-----------------------------------------------------------------------------------------------

function AutoFollowHelper:OnFollowCommand( wndHandler, wndControl, eMouseButton )
	ChatSystemLib.Command("/Follow")
	--insert /follow chat command here
end

function AutoFollowHelper:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "Auto Follow Window"})
end

-----------------------------------------------------------------------------------------------
-- AutoFollowHelper Instance
-----------------------------------------------------------------------------------------------
local AutoFollowHelperInst = AutoFollowHelper:new()
AutoFollowHelperInst:Init()
